import { AutoMap } from '@automapper/classes';
import {
  IsOptional,
  IsString,
  IsBoolean,
  IsDate,
  IsInt,
  IsEnum,
  ValidateNested,
  IsLongitude,
  IsLatitude,
} from 'class-validator';
import { Frequency } from '../entities/frequency.enum';
import { Transform, Type } from 'class-transformer';

export class CreateAdRequest {
  @IsString()
  @AutoMap()
  userUuid: string;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  driver?: boolean;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  passenger?: boolean;

  @Transform(({ value }) => Frequency[value - 1], { toClassOnly: true })
  @IsEnum(Frequency)
  @AutoMap()
  frequency: Frequency;

  @Type(() => Date)
  @IsDate()
  @IsOptional()
  @AutoMap()
  departure?: Date;

  @Type(() => Date)
  @IsDate()
  @IsOptional()
  @AutoMap()
  fromDate?: Date;

  @Type(() => Date)
  @IsDate()
  @IsOptional()
  @AutoMap()
  toDate?: Date;

  @IsDate()
  @IsOptional()
  @AutoMap()
  monTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  tueTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  wedTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  thuTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  friTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  satTime?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  sunTime?: string;

  @IsInt()
  @IsOptional()
  @AutoMap()
  monMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  tueMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  wedMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  thuMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  friMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  satMargin?: number;

  @IsInt()
  @IsOptional()
  @AutoMap()
  sunMargin?: number;

  @IsOptional()
  @IsInt()
  @AutoMap()
  seatsDriver?: number;

  @IsOptional()
  @IsInt()
  @AutoMap()
  seatsPassenger?: number;

  @IsOptional()
  @IsBoolean()
  @AutoMap()
  strict?: boolean;

  @ValidateNested({ each: true })
  @Transform(
    ({ value }) => {
      console.log('in dto');
      console.log(value);
      return value;
    },
    { toClassOnly: true },
  )
  @Type(() => AddressDTO)
  @AutoMap(() => [AddressDTO])
  addresses: AddressDTO[];
}

class AddressDTO {
  @IsInt()
  @AutoMap()
  position: number;

  @IsLongitude()
  @AutoMap()
  lon: number;

  @IsLatitude()
  @AutoMap()
  lat: number;

  @IsString()
  @IsOptional()
  @AutoMap()
  name?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  houseNumber?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  street?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  locality?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  postalCode?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  country?: string;

  @IsString()
  @IsOptional()
  @AutoMap()
  countryCode?: string;
}
